package com.phlx.anchorcollect.ble;

import static com.phlx.anchorcollect.Configs.uuid_service;
import static com.phlx.anchorcollect.Configs.uuid_write;

import android.app.Service;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.Intent;
import android.os.IBinder;
import android.os.SystemClock;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.phlx.anchorcollect.App;
import com.phlx.anchorcollect.Configs;
import com.phlx.anchorcollect.event.EventKey;
import com.phlx.anchorcollect.event.EventMsg;
import com.phlx.anchorcollect.event.EventTag;
import com.phlx.anchorcollect.event.EventUtils;
import com.phlx.anchorcollect.util.DataManageUtils;
import com.uhf.structures.OnBluetoothListener;

import org.greenrobot.eventbus.EventBus;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

import cn.wandersnail.ble.Connection;
import cn.wandersnail.ble.ConnectionState;
import cn.wandersnail.ble.Device;
import cn.wandersnail.ble.EasyBLE;
import cn.wandersnail.ble.EventObserver;
import cn.wandersnail.ble.Request;
import cn.wandersnail.ble.RequestBuilder;
import cn.wandersnail.ble.RequestBuilderFactory;
import cn.wandersnail.ble.callback.MtuChangeCallback;
import cn.wandersnail.ble.callback.NotificationChangeCallback;
import cn.wandersnail.commons.observer.Observe;
import cn.wandersnail.commons.poster.RunOn;
import cn.wandersnail.commons.poster.Tag;
import cn.wandersnail.commons.poster.ThreadMode;
import me.goldze.mvvmhabit.bus.RxBus;


/**
 * <pre>
 *     ble蓝牙service
 * </pre>
 */
public class BleService extends Service implements EventObserver {

    private static final String TAG = BleService.class.getSimpleName();
    private BleManagers bleManagers;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }


    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        EasyBLE.getInstance().registerObserver(this);
        bleManagers = BleManagers.from(this);
        EventUtils.post(new EventMsg(EventKey.KEY_BLUETOOTH_CONNECT_SUCCESS, null));
        Log.e(TAG, "onStartCommand: " + "服务开启了");
        return START_STICKY;

    }


    @Tag("onCharacteristicChanged")
    @RunOn(ThreadMode.BACKGROUND)
    @Observe
    @Override
    public void onCharacteristicChanged(Device device, UUID service, UUID characteristic, byte[] value) {
//        App.getInstance().getLinkage().pushRemoteRFIDData(value);
        String changedStr = byteToString(2, value);
        Log.e("Data:notify", value.length + "<--" + changedStr);
        if(value[1] == (byte) 0x0A ) {
            //设置model
            bleManagers.writeCharacteristic(uuid_service, uuid_write, new byte[]{(byte) 0xAA, 0x54, 0x01, (byte) 0xFF, 0x00});
            //告诉设备收到消息
            bleManagers.writeCharacteristic(uuid_service, uuid_write, new byte[]{(byte) 0xAA, (byte) 0x8A, 0x01, 0x35, 0x00});
            SystemClock.sleep(300);
            EventMsg eventMsg = new EventMsg();
            eventMsg.setAction(EventTag.EVENT_BLE_LEN);
            eventMsg.setObj(changedStr);
            RxBus.getDefault().post(eventMsg);
        }
    }


    /**
     * 写入状态监听
     *
     * @param request
     * @param value
     */
    @Tag("onCharacteristicWrite")
    @RunOn(ThreadMode.BACKGROUND)
    @Observe
    @Override
    public void onCharacteristicWrite(@NonNull Request request, @NonNull byte[] value) {
        Log.e("写入状态监听-", "成功写入:" + cn.wandersnail.commons.util.StringUtils.toHex(value, " "));
//        if (uuid_read != null && uuid_notify == null) {
//            bleManagers.readCharacteristic(uuid_service, uuid_read);
//        }
    }

    @Override
    public void onCharacteristicRead(@NonNull Request request, @NonNull byte[] value) {
        if (value.length > 0) {
            App.getInstance().getLinkage().pushRemoteRFIDData(value);
            //   Log.e("Data:read", value.length + "<--" + bytesToHexString(value, value.length));
        }
    }


    @Override
    public void onDestroy() {
        Log.e(TAG, "onDestroy: " + "服务挂掉了");
        EasyBLE.getInstance().unregisterObserver(this);
    }

    public static String bytesToHexString(byte src[], int length) {
        StringBuilder stringBuilder = new StringBuilder("");
        if (src == null || src.length <= 0 || length <= 0)
            return null;
        for (int i = 0; i < length && i < src.length; i++) {
            int v = src[i] & 0xff;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2)
                stringBuilder.append(0);
            stringBuilder.append(hv + " ");
        }
        return stringBuilder.toString().toUpperCase();
    }

    public static String byteToString(int length, byte[] bytes) {
        String data = "";
        byte[] result = new byte[length];
        try {
            System.arraycopy(bytes, 3, result, 0, length);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String byteArrayToString = DataManageUtils.byteArrayToString(result);
        int l = 0;
        if (!"ffff".equals(byteArrayToString)) {
            l = Integer.parseInt(Objects.requireNonNull(byteArrayToString), 16);
            double resultDouble = (double) l / 10;
            data = resultDouble + "";
        }
        return data;
    }


}