package com.phlx.anchorcollect.ui.setting;

import static com.phlx.anchorcollect.Configs.uuid_service;
import static com.phlx.anchorcollect.Configs.uuid_write;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.Application;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.databinding.ObservableArrayList;
import androidx.databinding.ObservableField;
import androidx.databinding.ObservableList;

import com.phlx.anchorcollect.App;
import com.phlx.anchorcollect.BR;
import com.phlx.anchorcollect.Configs;
import com.phlx.anchorcollect.R;
import com.phlx.anchorcollect.ble.BleManagers;
import com.phlx.anchorcollect.ble.OnBleSearchListener;
import com.phlx.anchorcollect.bt.BluetoothEntity;
import com.phlx.anchorcollect.bt.BtService;
import com.phlx.anchorcollect.data.Repository;
import com.phlx.anchorcollect.event.EventKey;
import com.phlx.anchorcollect.event.EventMsg;
import com.phlx.anchorcollect.event.EventTag;
import com.phlx.anchorcollect.event.EventUtils;
import com.phlx.anchorcollect.rfid.ModuleManager;
import com.phlx.anchorcollect.util.TextUtil;
import com.uhf.structures.FilterParams;
import com.uhf.structures.Rfid_Value;

import java.util.ArrayList;
import java.util.List;

import cn.wandersnail.ble.Connection;
import cn.wandersnail.ble.ConnectionState;
import cn.wandersnail.ble.Device;
import cn.wandersnail.ble.EasyBLE;
import cn.wandersnail.ble.EventObserver;
import cn.wandersnail.ble.Request;
import cn.wandersnail.ble.RequestBuilder;
import cn.wandersnail.ble.RequestBuilderFactory;
import cn.wandersnail.ble.RequestType;
import cn.wandersnail.ble.callback.MtuChangeCallback;
import cn.wandersnail.ble.callback.NotificationChangeCallback;
import cn.wandersnail.bluetooth.BTManager;
import cn.wandersnail.bluetooth.DiscoveryListener;
import cn.wandersnail.commons.observer.Observe;
import cn.wandersnail.commons.poster.RunOn;
import cn.wandersnail.commons.poster.Tag;
import cn.wandersnail.commons.poster.ThreadMode;
import cn.wandersnail.commons.util.ToastUtils;
import io.reactivex.disposables.Disposable;
import me.goldze.mvvmhabit.base.BaseViewModel;
import me.goldze.mvvmhabit.binding.command.BindingAction;
import me.goldze.mvvmhabit.binding.command.BindingCommand;
import me.goldze.mvvmhabit.bus.RxBus;
import me.goldze.mvvmhabit.bus.RxSubscriptions;
import me.goldze.mvvmhabit.bus.event.SingleLiveEvent;
import me.goldze.mvvmhabit.utils.SPUtils;
import me.tatarka.bindingcollectionadapter2.ItemBinding;

public class SettingVM extends BaseViewModel<Repository> implements DiscoveryListener, OnBleSearchListener, EventObserver {


    //给RecyclerView添加ObservableList
    public ObservableList<BluetoothItem> observaBluetoothList = new ObservableArrayList<>();
    //给RecyclerView添加ItemBinding
    public ItemBinding<BluetoothItem> itemBinding = ItemBinding.of(BR.viewModel, R.layout.item_bluetooth);
    //给BLE RecyclerView添加ObservableList
    public ObservableList<BleItem> observaBleList = new ObservableArrayList<>();
    //给RecyclerView添加ItemBinding
    public ItemBinding<BleItem> bleItemBinding = ItemBinding.of(BR.viewModel, R.layout.item_ble);

    private Device curDevice;

    private List<Device> bleList;
    ;

    private BleManagers bleManagers;

    private Handler bleHandler;

    private static final long SCAN_BLE_PERIOD = 5000;

    //ble  是否在扫描
    private boolean bleScanning = false;

    //选中的蓝牙设备信息
    public static BluetoothEntity bluetoothEntity;
    public static Device bleDevice;

    //当前蓝牙搜索集合
    private List<BluetoothEntity> bluetoothList;

    public SingleLiveEvent<Integer> btRequestEvent = new SingleLiveEvent<>();
    public SingleLiveEvent<Boolean> btParamEvent = new SingleLiveEvent<>();

    //错误提示
    public SingleLiveEvent<String> errorEvent = new SingleLiveEvent<>();
    //关闭页面
    public SingleLiveEvent<String> closeEvent = new SingleLiveEvent<>();
    //关闭对话框
    public SingleLiveEvent<String> closeDialogEvent = new SingleLiveEvent<>();

    //功率
    public ObservableField<String> power = new ObservableField<>("");
    public ObservableField<Boolean> inventoryTypeStatus = new ObservableField<>(false);


    private Disposable weightBus;

    public SettingVM(@NonNull Application application, Repository model) {
        super(application, model);

        initData();

    }


    private void initData() {
        bleManagers = BleManagers.from(getApplication());
        bleManagers.setListener(SettingVM.this);
        bleList = new ArrayList<>();
        observaBleList.clear();
        bleHandler = new Handler();
        EasyBLE.getInstance().registerObserver(this);

        inventoryTypeStatus.set(SPUtils.getInstance().getBoolean(Configs.SP_INVENTORY_STATUS, false));
    }

    //获取功率
    public BindingCommand getPowerOnClick = new BindingCommand(this::getAntennaPower);
    //设置功率
    public BindingCommand setPowerOnClick = new BindingCommand(this::setAntennaPower);


    /**
     * 获取天线功率
     */
    private void getAntennaPower() {
        if (!ModuleManager.initPower()) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("模块连接失败");
            ModuleManager.powerOff();
            return;
        }
        Rfid_Value power_value = new Rfid_Value();
        int powerStatue =
                App.getInstance().getLinkage().Radio_GetAntennaPower(power_value);
        if (powerStatue == 0) {
            String value = String.valueOf((power_value.value) / 10);
            power.set(value);
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("获取天线功率成功");
        } else {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("获取天线功率失败");
        }

    }

    /**
     * 设置天线功率
     */
    private void setAntennaPower() {
        if (!ModuleManager.initPower()) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("模块连接失败");
            ModuleManager.powerOff();
            return;
        }
        if (TextUtil.isSpace(power.get())) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("功率不能为空");
            return;
        }
        int status_power;
        int iu32 = (int) (Double.parseDouble(power.get()) * 10);
        if (iu32 < 0) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("参数不可用");
            return;
        }
        status_power = App.getInstance().getLinkage().Radio_SetAntennaPower(iu32);

        if (status_power == 0) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("功率修改成功");

        } else if (status_power == -1000) {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("正在盘点");
        } else {
            me.goldze.mvvmhabit.utils.ToastUtils.showShort("设置功率失败");
        }
    }

    public void initBt(Context context) {

        bluetoothList = new ArrayList<>();
        BTManager.isDebugMode = true;
        BTManager.getInstance().addDiscoveryListener(this);
        btParamEvent.call();
    }

    public BindingCommand OnRefreshClickCommand = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            BTManager.getInstance().stopDiscovery();
            doStartDiscovery();
        }
    });

    private void doStartDiscovery() {
        observaBluetoothList.clear();
        bluetoothList.clear();
        BTManager.getInstance().startDiscovery();
    }

    public BindingCommand OnStopClickCommand = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            BTManager.getInstance().stopDiscovery();
        }
    });

    public BindingCommand OnBleRefreshClickCommand = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            bleManagers.disCurConnectBLE();
            if(bleScanning){
                return;
            }
            clearBleList();
            Log.e("", "OnBleRefreshClickCommand--bleList--"+System.identityHashCode(bleList));
            if (!bleList.isEmpty()) {
                bleList.clear();
            }
            observaBleList.clear();
            //延时停止
            bleHandler.postDelayed(() -> {
                bleScanning = false;
                dismissDialog();
                bleManagers.cancelBLESearch();
            }, SCAN_BLE_PERIOD);
            bleManagers.searchBleDevices();
        }
    });

    public void clearBleList(){
        if (!bleList.isEmpty()) {
            bleList.clear();
        }
    }

    public BindingCommand OnBleStopClickCommand = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            bleManagers.cancelBLESearch();
        }
    });

    public BindingCommand inventoryTypeChoiceOnClick = new BindingCommand(new BindingAction() {
        @Override
        public void call() {
            Configs.inventoryStatus = inventoryTypeStatus.get();
            if (inventoryTypeStatus.get()) {
                setGeneralInventoryFilter();
            } else {
                setSpecialInventoryFilter();
            }
            SPUtils.getInstance().put(Configs.SP_INVENTORY_STATUS, Configs.inventoryStatus);
        }
    });

    public void itemOnClick(BluetoothEntity btEntity) {
        BTManager.getInstance().stopDiscovery();
        stopBtService();
        bluetoothEntity = btEntity;

        showDialog("正在链接蓝牙...");

        Intent intent = new Intent(getApplication(), BtService.class);
        intent.putExtra("device", bluetoothEntity.getBluetoothDevice());
        if (!isBtServiceRunning(getApplication()))
            getApplication().startService(intent);
    }

    public void bleItemOnClick(Device device) {
//        bleManagers.disConnectBLE(device);
        stopBtService();
        bleDevice = device;

        showDialog("正在链接BLE蓝牙...");
        bleManagers.connectBleDevice(device);
    }


    @Override
    public void onDiscoveryStart() {

    }

    @Override
    public void onDiscoveryStop() {

    }

    @Override
    public void onDeviceFound(@NonNull BluetoothDevice device, int rssi) {
        if (!TextUtils.isEmpty(device.getName())) {
            BluetoothEntity bluetooth = new BluetoothEntity();
            bluetooth.setName(device.getName());
            bluetooth.setAddress(device.getAddress());
            bluetooth.setRssi(rssi);
            bluetooth.setBluetoothDevice(device);
            if (!bluetoothList.contains(bluetooth)) {
                bluetoothList.add(bluetooth);
                BluetoothItem bluetoothItem = new BluetoothItem(this, bluetooth);
                observaBluetoothList.add(bluetoothItem);
            }
        }
    }

    /**
     * 设置为标签最多的盘点
     *
     * @return
     */
    private int setSpecialInventoryFilter() {
        int tsh = 10;
        int maxRepeat = 1;
        int cacheTime = 0;
        int maxMonitorTagInRangeTimeMs = 400;
        int maxMonitorTagOutRangeTimeMs = 200;
        int filterMode = 2;
        maxRepeat |= (filterMode << 14);
        return setInventoryFilter(tsh, maxRepeat, cacheTime, maxMonitorTagInRangeTimeMs, maxMonitorTagOutRangeTimeMs);
    }

    /**
     * 设置为普通盘点
     *
     * @return
     */
    private int setGeneralInventoryFilter() {
        int tsh = 0;
        int maxRepeat = 1;
        int cacheTime = 0;
        int maxMonitorTagInRangeTimeMs = 0;
        int maxMonitorTagOutRangeTimeMs = 0;
        int filterMode = 0;
        maxRepeat |= (filterMode << 14);
        return setInventoryFilter(tsh, maxRepeat, cacheTime, maxMonitorTagInRangeTimeMs, maxMonitorTagOutRangeTimeMs);
    }

    /**
     * 设置盘点过滤门限值
     *
     * @param threshold      过滤门限，最大值为 10
     * @param maxRepeatTimes 最大上报次数
     * @param maxCacheTimeMs 最大缓存时间(ms)
     * @return 0--成功 非零--失败
     */
    public int setInventoryFilter(int threshold, int maxRepeatTimes, int maxCacheTimeMs, int maxMonitorTagInRangeTimeMs, int maxMonitorTagOutRangeTimeMs) {
        FilterParams filterParams = new FilterParams();
        filterParams.setThreshold(threshold);
        filterParams.setFilerFlag(maxRepeatTimes);
        filterParams.setMaxCacheTimeMs(maxCacheTimeMs);
        filterParams.setMaxMonitorTagInRangeTimeMs(maxMonitorTagInRangeTimeMs);
        filterParams.setMaxMonitorTagOutRangeTimeMs(maxMonitorTagOutRangeTimeMs);
        return App.getInstance().getLinkage().setInventoryFilter(filterParams);
    }


    @Override
    public void onDiscoveryError(int errorCode, @NonNull String errorMsg) {
        switch (errorCode) {
            case DiscoveryListener.ERROR_LACK_LOCATION_PERMISSION://缺少定位权限
                ToastUtils.showShort("搜索出错：缺少定位权限" + errorCode);
                break;
            case DiscoveryListener.ERROR_LOCATION_SERVICE_CLOSED://位置服务未开启
                ToastUtils.showShort("搜索出错：位置服务未开启" + errorCode);
                break;
            case DiscoveryListener.ERROR_LACK_SCAN_PERMISSION://缺少搜索权限
                ToastUtils.showShort("搜索出错：缺少搜索权限" + errorCode);
                break;
            case DiscoveryListener.ERROR_SCAN_FAILED://搜索失败
                ToastUtils.showShort("搜索出错：" + errorCode);
                break;
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (BTManager.getInstance().isInitialized()) {
            BTManager.getInstance().stopDiscovery();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d("Main", "onResume");
        if (BTManager.getInstance().isInitialized()) {
            if (BTManager.getInstance().isBluetoothOn()) {
//                doStartDiscovery();
            } else {
                //未开启蓝牙
                btRequestEvent.call();
            }
        }
    }

    /**
     * 判断服务是否在运行中
     *
     * @param context 即为Context对象
     * @return 是否在运行中
     */
    private boolean isBtServiceRunning(Context context) {
        if (context != null) {
            ActivityManager activityManager
                    = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            ArrayList<ActivityManager.RunningServiceInfo> runningServiceInfoList
                    =
                    (ArrayList<ActivityManager.RunningServiceInfo>) activityManager.getRunningServices(100);
            for (ActivityManager.RunningServiceInfo runningServiceInfo : runningServiceInfoList) {
                if ("BtService".equals(runningServiceInfo.service.getClassName())) {
                    return true;
                }
            }
        } else {
            return false;
        }
        return false;
    }


    public void stopBtService() {
        if (isBtServiceRunning(getApplication())) {
            Intent intent = new Intent(getApplication(), BtService.class);
            getApplication().stopService(intent);
        }
    }


    @Override
    public void registerRxBus() {
        super.registerRxBus();
        weightBus =
                RxBus.getDefault().toObservable(EventMsg.class).subscribe(eventType -> {
                    if (eventType.getAction().equals(EventTag.EVENT_BT_SUCC)) {
                        ToastUtils.showShort("蓝牙链接成功");
                        myHandler.sendEmptyMessage(0);
//                        closeEvent.call();
                    } else if (eventType.getAction().equals(EventTag.EVENT_BT_FAIL)) {
                        myHandler.sendEmptyMessage(0);
                        ToastUtils.showShort("蓝牙链接失败");
                    }
                });
        RxSubscriptions.add(weightBus);
    }

    @Override
    public void removeRxBus() {
        super.removeRxBus();
        RxSubscriptions.remove(weightBus);
    }

    @SuppressLint("HandlerLeak")
    private Handler myHandler = new Handler() {
        @Override
        public void handleMessage(@NonNull Message msg) {
            super.handleMessage(msg);
            dismissDialog();
            finish();
        }
    };


    @Override
    public void onStartDiscovery() {
        bleScanning = true;
        Log.e("", "onStartDiscovery--bleScanning--"+bleScanning);
    }

    @Override
    public void onStopDiscovery() {
        bleScanning = false;
        Log.e("", "onStopDiscovery--bleScanning--"+bleScanning);
    }

    @Override
    public void onBleDeviceFound(Device device) {

        Log.e("", "onBleDeviceFound--bleList--"+System.identityHashCode(bleList));
        if (!bleList.contains(device) && !TextUtils.isEmpty(device.getName())) {
            bleList.add(device);

            observaBleList.add(new BleItem(this, device));
        }
    }

    @Override
    public void onBleError(String e) {

    }

    @Observe
    @Override
    public void onNotificationChanged(@NonNull Request request, boolean isEnabled) {
        if (request.getType() == RequestType.SET_NOTIFICATION) {
            if (isEnabled) {
                bleManagers.startBleService();
                try {
                    Thread.sleep(1000);
//                    getVersion();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            } else {

            }
        } else {
            if (isEnabled) {
                bleManagers.startBleService();
            } else {
            }
        }
    }

    /**
     * 连接状态 监听
     *
     * @param device
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Tag("onConnectionStateChanged")
    @Observe
    @RunOn(ThreadMode.MAIN)
    @Override
    public void onConnectionStateChanged(@NonNull Device device) {
        ConnectionState connectionState = device.getConnectionState();
        if (connectionState == ConnectionState.SCANNING_FOR_RECONNECTION) {
        } else if (connectionState == ConnectionState.DISCONNECTED) {
            EventUtils.post(new EventMsg(EventKey.KEY_BLUETOOTH_CONNECT_FAILED, null));
            EventUtils.post(new EventMsg<>(EventKey.KEY_INVENTORY_STOP, null));
        } else if (connectionState == ConnectionState.CONNECTING) {
        } else if (connectionState == ConnectionState.CONNECTED) {
            //TODO
        } else if (connectionState == ConnectionState.SERVICE_DISCOVERING) {
        } else if (connectionState == ConnectionState.SERVICE_DISCOVERED) {
            List<BluetoothGattService> services = bleManagers.getConnection().getGatt().getServices();
            for (BluetoothGattService service : services) {
                List<BluetoothGattCharacteristic> characteristics = service.getCharacteristics();
                for (BluetoothGattCharacteristic characteristic : characteristics) {
                    String characteristicType = getCharacteristicType(characteristic);
                    if ("0000fff3-0000-1000-8000-00805f9b34fb".equals(characteristic.getUuid().toString())) {
                        uuid_service = service.getUuid();
                        Log.e("uuid_service", "uuid_service:" + uuid_service.toString());
                        Configs.mNotifyCharacteristic3 = characteristic;
                        uuid_write = characteristic.getUuid();
                        Log.e("uuid_write", "uuid_write:" + uuid_write);
                        if ((characteristic.getProperties() & BluetoothGattCharacteristic.PROPERTY_WRITE) > 0 ||
                                (characteristic.getProperties() & BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE) > 0) {
                            // 支持写操作
                        } else {
                            Log.e("BLE", "该 Characteristic 不支持写入");
                        }
                    } else if ("0000fff6-0000-1000-8000-00805f9b34fb".equals(characteristic.getUuid().toString())) {

                        Configs.mNotifyCharacteristic6 = characteristic;
                        Configs.uuid_notify = characteristic.getUuid();
                        Log.e("uuid_notify", "uuid_notify:" + Configs.uuid_notify);
                    }
                }
            }
            //设置MTU
            Connection connection = EasyBLE.getInstance().getConnection(device);
            RequestBuilder<MtuChangeCallback> builder = new RequestBuilderFactory().getChangeMtuBuilder(512);
            Request request = builder.setCallback(new MtuChangeCallback() {
                @Override
                public void onRequestFailed(@NonNull Request request, int failType, @Nullable Object value) {
                    ToastUtils.showShort("体尺设备连接失败！");
                }

                @Override
                public void onMtuChanged(@NonNull Request request, int mtu) {
                    Log.e("EasyBLE", "MTU修改成功，新值：" + mtu);
                    setNotification();
                    closeDialogEvent.postValue("");
                    //设置连接模式为长，值为0.并且开始接受传值
                    bleManagers.writeCharacteristic(uuid_service, uuid_write, new byte[]{(byte) 0xAA, 0x54, 0x01, (byte) 0xFF, 0x00});
                    ToastUtils.showShort("体尺设备连接成功！");
                    closeEvent.postValue("");
                }

            }).build();
            connection.execute(request);
        }
    }

    private String getCharacteristicType(BluetoothGattCharacteristic characteristic) {
        StringBuilder sb = new StringBuilder();
        int[] properties = new int[]{BluetoothGattCharacteristic.PROPERTY_WRITE, BluetoothGattCharacteristic.PROPERTY_INDICATE, BluetoothGattCharacteristic.PROPERTY_NOTIFY, BluetoothGattCharacteristic.PROPERTY_READ, BluetoothGattCharacteristic.PROPERTY_SIGNED_WRITE, BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE};
        String[] propertyStrs = new String[]{"WRITE", "INDICATE", "NOTIFY", "READ", "SIGNED_WRITE", "WRITE_NO_RESPONSE"};
        for (int i = 0; i < properties.length; i++) {
            int property = properties[i];
            if ((characteristic.getProperties() & property) != 0) {
                if (sb.length() != 0) {
                    sb.append(", ");
                }
                sb.append(propertyStrs[i]);
            }
        }
        return sb.toString();
    }


    private void setNotification() {
        boolean isEnabled = bleManagers.getConnection().isNotificationOrIndicationEnabled(uuid_service, Configs.uuid_notify);

        RequestBuilder<NotificationChangeCallback> builder = new RequestBuilderFactory().getSetNotificationBuilder(uuid_service, Configs.uuid_notify, !isEnabled);
        builder.build().execute(bleManagers.getConnection());
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        bleManagers.removeListener();
    }
}
